-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity AppleShifter Is
    Port (
        clk28         : In  Std_Logic;
        AppleII_Reset : In  Std_Logic;
        do_shift      : In  Std_Logic;
        odd_dot       : In  Std_logic;
        LDPS          : In  Std_Logic;
        TextIn        : In  Std_Logic;
        Mixed         : In  Std_Logic;
        AltCharSet    : In  Std_Logic;
        HiRes         : In  Std_Logic;
        flash_count4  : In  Std_Logic;
        HBL           : In  Std_Logic;
        VBL           : In  Std_Logic;
        aux_LDPS      : In  Std_Logic;
        mode_80COL    : In  Std_Logic;
        ram_data      : In  Std_Logic_Vector(15 Downto 0);
        CounterY      : In  Std_Logic_Vector(9 Downto 0);
        pixelout      : Out Std_Logic;
        GR            : Out Std_Logic;
        char_ROM_addr : Out Std_Logic_Vector(9 Downto 0);
        debug8bits    : Out Std_Logic_Vector(7 Downto 0);
        char_ROM_data : In  Std_Logic_Vector(7 Downto 0);
        vga_R_r       : Out Std_Logic_Vector(7 Downto 0);
        vga_G_r       : Out Std_Logic_Vector(7 Downto 0);
        vga_B_r       : Out Std_Logic_Vector(7 Downto 0)

    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of AppleShifter Is
-- ----------------------------------------------------------------------------
Component lores_color_map Is
    Port (
        nibble  : In  Std_Logic_Vector(3 Downto 0);
        RGB_val : Out Std_Logic_Vector(11 Downto 0)
    );
End Component;

    Signal flashing           : Std_Logic;
    Signal hicolor            : Std_Logic_Vector(11 Downto 0);
    Signal flash              : Std_Logic;
    Signal BLANK              : Std_Logic;
    Signal HBL1               : Std_Logic;
    Signal VBL1               : Std_Logic;
    Signal GR_Sig             : Std_Logic;
    Signal shiftreg           : Std_Logic_Vector(6 Downto 0);
    Signal inverse            : Std_Logic;
    Signal final_char_code    : Std_Logic_Vector(6 Downto 0);
    Signal char_shift_data    : Std_Logic_Vector(7 Downto 0);
    Signal lowres_color_val   : Std_Logic_Vector(11 Downto 0);
    Signal nibble             : Std_Logic_Vector(3 Downto 0);
    Signal low_color_nibble   : Std_Logic_Vector(3 Downto 0);
    Signal four_pixels        : Std_Logic_Vector(3 Downto 0);
    Signal hi_R               : Std_Logic_Vector(3 Downto 0);
    Signal hi_G               : Std_Logic_Vector(3 Downto 0);
    Signal hi_B               : Std_Logic_Vector(3 Downto 0);
    Signal pixelout_sig       : Std_Logic;
    Signal mono_pixel         : Std_Logic_Vector(7 Downto 0);
    Signal hi_R8              : Std_Logic_Vector(7 Downto 0);
    Signal hi_G8              : Std_Logic_Vector(7 Downto 0);
    Signal hi_B8              : Std_Logic_Vector(7 Downto 0);
    Signal vga_R              : Std_Logic_Vector(7 Downto 0);
    Signal vga_G              : Std_Logic_Vector(7 Downto 0);
    Signal vga_B              : Std_Logic_Vector(7 Downto 0);
    Signal aux_video_data     : Std_Logic_Vector(7 Downto 0);
    Signal latched_video_data : Std_Logic_Vector(7 Downto 0);
    Signal char_code          : std_Logic_Vector(7 Downto 0);
    Signal video_shift_data   : Std_Logic_Vector(7 Downto 0);
    Signal CounterY_0_del     : Std_Logic;
    Signal odd                : Std_Logic;
    Signal high_bit           : Std_Logic;
    Signal load_shifter       : Std_Logic;
    Signal aux_load_shifter   : Std_Logic;
    Signal last_pixel         : Std_Logic;
    Signal char_code7         : Std_Logic;
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    flashing <= flash_count4 When flash='1' Else '0';
    debug8bits <= hicolor(7 Downto 0);
    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            BLANK <= (HBL1 Or VBL1);
            If (LDPS='1') Then
                GR_Sig <= Not (TextIn Or (Mixed And CounterY(6) And CounterY(8)));
            End If;
        End If;
    End Process;
    GR <= GR_Sig;
    -- ------------------------------------------------------------------------
    char_code7 <= char_code(6) And char_code(7);
    final_char_code <= char_code(6 Downto 0) When AltCharSet='1' Else
                       char_code7 & char_code(5 Downto 0);

    pixelout <= pixelout_sig;
    pixelout_sig <= shiftreg(0) Xor ((inverse Xor flashing) And Not GR_Sig);
    char_ROM_addr <= final_char_code & CounterY(3 Downto 1);
    char_shift_data <= char_ROM_data;
    -- ------------------------------------------------------------------------
    nibble <= four_pixels When HiRes='1' Else low_color_nibble;

    U_LoresColourMap : lores_color_map Port Map (
        nibble => nibble,
        RGB_val => lowres_color_val
    );
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    hi_R <= hicolor(11 Downto 8) When shiftreg(0)='1' Else "0000";
    hi_G <= hicolor( 7 Downto 4) When shiftreg(0)='1' Else "0000";
    hi_B <= hicolor( 3 Downto 0) When shiftreg(0)='1' Else "0000";
    mono_pixel <= pixelout_sig & pixelout_sig & pixelout_sig & pixelout_sig & pixelout_sig & pixelout_sig & pixelout_sig & pixelout_sig;
    hi_R8 <= hi_R & hi_R(0) & hi_R(0) & hi_R(0) & hi_R(0);
    hi_G8 <= hi_G & hi_G(0) & hi_G(0) & hi_G(0) & hi_G(0);
    hi_B8 <= hi_B & hi_B(0) & hi_B(0) & hi_B(0) & hi_B(0);
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            vga_R_r <= vga_R;
            vga_G_r <= vga_G;
            vga_B_r <= vga_B;
        End If;
    End Process;
    -- ------------------------------------------------------------------------
    vga_R <= "00000000" When BLANK='1'  Else
             mono_pixel When GR_Sig='0' Else
             hi_R8      When HiRes='1'  Else
             lowres_color_val(11 Downto 8) & "0000";

    vga_G <= "00000000" When BLANK='1'  Else
             mono_pixel When GR_Sig='0' Else
             hi_G8      When HiRes='1'  Else
             lowres_color_val( 7 Downto 4) & "0000";

    vga_B <= "00000000" When BLANK='1'  Else
             mono_pixel When GR_Sig='0' Else
             hi_B8      When HiRes='1'  Else
             lowres_color_val( 3 Downto 0) & "0000";

    char_code <= ram_data(15 Downto 8) When (mode_80COL='1' And LDPS='1') Else
                 latched_video_data    When (mode_80COL='1')              Else
                 ram_data(7 Downto 0);
    video_shift_data <= latched_video_data When GR_Sig='1' Else char_shift_data;
    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            four_pixels <= four_pixels(2 Downto 0) & shiftreg(0);
            If (GR_Sig='1' And HiRes='1') Then
                latched_video_data <= ram_data(7 Downto 0);
            Else
                If (LDPS='1') Then
                    If CounterY(3)='1' Then low_color_nibble <= ram_data(7 Downto 4);
                                       Else low_color_nibble <= ram_data(3 Downto 0);
                    End If;
                    aux_video_data     <= ram_data(15 Downto 8);
                    latched_video_data <= ram_data(7 Downto 0);
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If (LDPS='1') Then
                CounterY_0_del <= CounterY(0);
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(last_pixel, shiftreg, odd, high_bit)
    Begin
        If (shiftreg(1) = '1' Or last_pixel='1') Then
            hicolor <= x"FFF";
        Else
            If (odd='1') Then
                If (high_bit='1') Then
                    hicolor <= x"22F"; -- med blue
                Else
                    hicolor <= x"d2d"; -- purple
                End If;
            Else
                If (high_bit='1') Then
                    hicolor <= x"d03"; -- red
                Else
                    hicolor <= x"1d0"; -- green
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            load_shifter <= LDPS;
            aux_load_shifter <= aux_LDPS;
            odd <= Not (odd_dot Xor CounterY_0_del);
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If (LDPS='1') Then
                HBL1 <= HBL;
--                HBL2 <= HBL1;
                VBL1 <= VBL;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If (load_shifter='1' Or aux_load_shifter='1') Then
                shiftreg <= video_shift_data(6 Downto 0);
                high_bit <= video_shift_data(7);
                If (load_shifter='0' Or mode_80COL='0') Then
                    inverse <= Not latched_video_data(7);
                    If AltCharSet='1' Then
                        flash <= '0';
                    Else
                        flash <= latched_video_data(6) And Not latched_video_data(7);
                    End If;
                Else
                    inverse <= Not aux_video_data(7);
                    If AltCharSet='1' Then
                        flash <= '0';
                    Else
                        flash <= aux_video_data(6) And Not aux_video_data(7);
                    End If;
                End iF;
            Else
                If (do_shift='1') Then
                    shiftreg <= '0' & shiftreg(6 Downto 1);
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If (do_shift='1') Then
                last_pixel <= shiftreg(0);
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
