-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
Use     IEEE.Std_Logic_Unsigned.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity AppleVideo Is
    Port (
        clk28         : In    Std_Logic;
        AppleII_Reset : In    Std_Logic;
        rw_6502       : In    Std_Logic;
        ram_data      : Inout Std_logic_Vector(15 Downto 0);
        char_ROM_data : In    Std_Logic_Vector( 7 Downto 0);
--        reset_in      : In    Std_Logic;
        do_6502       : In    Std_Logic_Vector( 7 Downto 0);
        cpu_addr      : In    Std_Logic_Vector(15 Downto 0);
        vga_R         : Out   Std_Logic_Vector( 7 Downto 0);
        vga_G         : Out   Std_Logic_Vector( 7 Downto 0);
        vga_B         : Out   Std_Logic_Vector( 7 Downto 0);
        vga_HS        : Out   Std_Logic;
        vga_VS        : out   Std_Logic;
        n_OE          : Out   Std_Logic;
        n_WE          : Out   Std_Logic;
        n_CS          : Out   Std_Logic;
        cpuclk_6502   : Out   Std_Logic;
        INTCXROM      : Out   Std_Logic;
        INTC8ROM      : Out   Std_Logic;
        SLOTC3ROM     : Out   Std_Logic;
        mode_80STORE  : Out   Std_Logic;
        BusCycle      : Out   Std_Logic_Vector( 4 Downto 0);
        char_ROM_addr : Out   Std_Logic_Vector( 9 Downto 0);
        RAM_address   : Out   Std_Logic_Vector(17 Downto 0);
        MMU_bit       : Out   Std_Logic;
        RAMRD         : Out   Std_Logic;
        RAMWRT        : Out   Std_Logic;
        disk_cycle    : Out   Std_Logic;
        card_ram_we   : Out   Std_Logic;
        card_ram_rd   : Out   Std_Logic;
        bank1         : Out   Std_Logic;
        spk           : Out   Std_Logic;
        io_access     : Out   Std_Logic;
        PAGE2         : Out   Std_Logic;
        ram_ub        : Out   Std_Logic;
        ram_lb        : Out   Std_Logic;
        vga_clk       : Out   Std_Logic;
        ram_d_l       : Out   Std_Logic_Vector(7 Downto 0)
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of AppleVideo Is
-- ----------------------------------------------------------------------------

Component RamCard Is
    Port (
        mclk28      : In  Std_Logic;
        reset_in    : In  Std_Logic;
        strobe      : In  Std_Logic;
        addr        : In  Std_Logic_Vector(15 Downto 0);
        ram_addr    : Out Std_Logic_Vector(15 Downto 0);
        we          : In  Std_Logic;
        card_ram_we : Out Std_Logic;
        card_ram_rd : Out Std_Logic;
        bank1       : Out Std_Logic
    );
End Component;

Component AppleShifter Is
    Port (
        clk28         : In  Std_Logic;
        AppleII_Reset : In Std_Logic;
        do_shift      : In  Std_Logic;
        odd_dot       : In  Std_logic;
        LDPS          : In  Std_Logic;
        TextIn        : In  Std_Logic;
        Mixed         : In  Std_Logic;
        AltCharSet    : In  Std_Logic;
        HiRes         : In  Std_Logic;
        flash_count4  : In  Std_Logic;
        HBL           : In  Std_Logic;
        VBL           : In  Std_Logic;
        aux_LDPS      : In  Std_Logic;
        mode_80COL    : In  Std_Logic;
        ram_data      : In  Std_Logic_Vector(15 Downto 0);
        CounterY      : In  Std_Logic_Vector(9 Downto 0);
        pixelout      : Out Std_Logic;
        GR            : Out Std_Logic;
        char_ROM_addr : Out Std_Logic_Vector(9 Downto 0);
        debug8bits    : Out Std_Logic_Vector(7 Downto 0);
        char_ROM_data : In  Std_Logic_Vector(7 Downto 0);
        vga_R_r       : Out Std_Logic_Vector(7 Downto 0);
        vga_G_r       : Out Std_Logic_Vector(7 Downto 0);
        vga_B_r       : Out Std_Logic_Vector(7 Downto 0)

    );
End Component;

Component AppleVideoTiming Is
    Port (
        clk28        : In  Std_Logic;
        AppleII_Reset : IN Std_Logic;
        vga_HS       : Out Std_Logic;
        vga_VS       : Out Std_Logic;
        HBL          : Out Std_Logic;
        VBL          : Out Std_Logic;
        LDPS         : Out Std_Logic;
        BusCycle     : out Std_Logic_Vector(4 Downto 0);
        flash_count4 : Out Std_Logic;
        CounterX     : Out Std_Logic_Vector(6 Downto 0);
        CounterY     : Out Std_Logic_Vector(9 Downto 0)
    );
End Component;

    Signal speaker          : Std_Logic;
    Signal auxilary_mem     : Std_Logic;
    Signal BusCycle_Sig     : Std_Logic_Vector(4 Downto 0);
    Signal CounterX         : Std_Logic_Vector(6 Downto 0);
    Signal CounterY         : Std_Logic_Vector(9 Downto 0);
    Signal GR               : Std_Logic;
    Signal HBL              : Std_Logic;
    Signal LDPS             : Std_Logic;
    Signal VBL              : Std_Logic;
    Signal flash_count4     : Std_Logic;
    Signal pixelout         : Std_Logic;
    Signal mem_rw_6502      : Std_Logic;
    Signal BusCycleIs5      : Std_Logic;
    Signal BusCycleIs6      : Std_Logic;
    Signal BusCycleIs7      : Std_Logic;
    Signal AltCharSet       : Std_Logic;
    Signal mode_80COL       : Std_Logic;
    Signal TextIn           : Std_Logic;
    Signal HiRes            : Std_Logic;
    Signal Mixed            : Std_Logic;
    Signal address_is_rom   : Std_Logic;
    Signal address_is_ram   : Std_Logic;
    Signal X_PAGE2          : Std_Logic;
    Signal PAGE2_Sig        : Std_Logic;
    Signal aux_LDPS         : Std_Logic;
    Signal BusCycleIs10     : Std_Logic;
    Signal BusCycleIs24     : Std_Logic;
    Signal high_vid_addr : Std_Logic_Vector(6 Downto 0);
    Signal NotX_PAGE2       : Std_Logic;
    Signal ram_data_low : Std_Logic_Vector(7 Downto 0);
    Signal ram_data_hi8 : Std_Logic_Vector(7 Downto 0);
    Signal cpu_access_time : Std_Logic;
    Signal BusCycleIs8 : Std_Logic;
    Signal addr_sum : Std_Logic_Vector(3 Downto 0);
    Signal apple_video_addr : Std_Logic_Vector(15 Downto 0);
--    Signal linear_address : Std_Logic_Vector(15 Downto 0);
    Signal final_ram_addr : Std_Logic_Vector(15 Downto 0);
    Signal memory_access_time : Std_Logic;
    Signal NotCounterX5 : Std_Logic;
    Signal BusCycleIs2 : Std_Logic;
    Signal BusCycleIs3 : Std_Logic;
    Signal BusCycleIs16 : Std_Logic;
    Signal BusCycleIs17 : Std_Logic;
--    Signal do_store : Std_Logic;
    Signal do_store_m : Std_Logic;
    Signal cpu_ram_oe : Std_logic;
--    Signal cpu_ram_We : Std_Logic;
    Signal card_ram_rd_sig : Std_logic;
    Signal ram_card_read : Std_Logic;
    Signal ram_card_write : Std_Logic;
    Signal card_ram_we_sig : Std_Logic;
    Signal low200 : Std_Logic;
    Signal c000_up : Std_Logic;
    Signal text_page0 : Std_Logic;
    Signal hgr_page0 : Std_Logic;
    Signal video_memory : Std_Logic;
    Signal ALTZP : std_Logic;
    Signal readback_mmu : Std_Logic;
--    Signal p : Std_Logic;
    Signal RAM_access : Std_Logic;
    Signal Notrw_6502 : Std_Logic;
    Signal doshift : Std_Logic;
    Signal BusCycleIs1 : Std_Logic;
    Signal mode_80STORE_Sig : Std_Logic;
    Signal ram_lb_Sig : Std_Logic;
    Signal RAMWRT_Sig : Std_Logic;
    Signal RAMRD_Sig : Std_Logic;
    Signal bank1_sig : Std_Logic;
    Signal INTCXROM_Sig : Std_Logic;
    Signal SLOTC3ROM_Sig : Std_Logic;
    Signal io_access_sig : Std_Logic;
    Signal BusCycleIs0 : Std_Logic;
    Signal odd_dot : Std_Logic;
    Signal AuxClk : Std_Logic;
    Signal ClockCounter : Std_Logic_Vector(4 Downto 0);
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------
    vga_clk      <= clk28;
    mode_80STORE <= mode_80STORE_Sig;
    card_ram_we  <= card_ram_we_Sig;
    ram_lb       <= ram_lb_Sig;
    RAMWRT       <= RAMWRT_Sig;
    RAMRD        <= RAMRD_Sig;
    bank1        <= bank1_sig;
    card_ram_rd  <= card_ram_rd_sig;
    INTCXROM     <= INTCXROM_Sig;
    SLOTC3ROM    <= SLOTC3ROM_Sig;
    io_access    <= io_access_sig;
    BusCycle     <= BusCycle_Sig;
    -- ------------------------------------------------------------------------
    BusCycleIs0  <= '1' When BusCycle_Sig="00000" Else '0';
    BusCycleIs1  <= '1' When BusCycle_Sig="00001" Else '0';
    BusCycleIs2  <= '1' When BusCycle_Sig="00010" Else '0';
    BusCycleIs3  <= '1' When BusCycle_Sig="00011" Else '0';
    BusCycleIs5  <= '1' When BusCycle_Sig="00101" Else '0';
    BusCycleIs6  <= '1' When BusCycle_Sig="00110" Else '0';
    BusCycleIs7  <= '1' When BusCycle_Sig="00111" Else '0';
    BusCycleIs8  <= '1' When BusCycle_Sig="01000" Else '0';
    BusCycleIs10 <= '1' When BusCycle_Sig="01010" Else '0';
    BusCycleIs16 <= '1' When BusCycle_Sig="10000" Else '0';
    BusCycleIs17 <= '1' When BusCycle_Sig="10001" Else '0';
    BusCycleIs24 <= '1' When BusCycle_Sig="11000" Else '0';
    mem_rw_6502  <= Not ((BusCycleIs5 Or BusCycleIs6 Or BusCycleIs7) And Not rw_6502);

    address_is_rom <= cpu_addr(15) And cpu_addr(14) And (cpu_addr(13) Or cpu_addr(12));
    address_is_ram <= Not (cpu_addr(15) And cpu_addr(14));
    PAGE2          <= PAGE2_Sig;
    X_PAGE2        <= PAGE2_Sig And Not mode_80STORE_Sig;

    aux_LDPS       <= (BusCycleIs10 Or BusCycleIs24) And mode_80COL;
    NotX_PAGE2     <= Not X_PAGE2;
    high_vid_addr  <= "00" & X_PAGE2 & NotX_PAGE2 & CounterY(3 Downto 1) When (HiRes='1' And GR='1') Else
                       "00000" & X_PAGE2 & NotX_PAGE2;
    spk <= speaker;

    ram_data_low <= do_6502 When (mem_rw_6502='0' And auxilary_mem='0' And (address_is_rom='0' Or card_ram_we_Sig='1')) Else
                    (Others=>'Z');
    ram_data_hi8 <= do_6502 When (mem_rw_6502='0' And auxilary_mem='1' And (address_is_rom='0' Or card_ram_we_Sig='1')) Else
                    (Others=>'Z');


    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If (BusCycleIs7='1') Then
                If ram_lb_Sig='1' Then
                    ram_d_l <= ram_data(15 Downto 8);
                Else
                    ram_d_l <= ram_data(7 Downto 0);
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    ram_data        <= ram_data_hi8 & ram_data_low;
    cpu_access_time <= BusCycleIs7 Or BusCycleIs8;
    NotCounterX5    <= Not CounterX(5);
    addr_sum        <= (NotCounterX5 & NotCounterX5 & CounterX(4) & CounterX(3)) +
                       ( CounterY(8) &  CounterY(7) & CounterY(8) & CounterY(7)) + 1;

    apple_video_addr <= high_vid_addr(5 Downto 0) & CounterY(6 Downto 4) & addr_sum & CounterX(2 Downto 0); --RAF!!!!!!!!!!!!
--    cpuclk_6502      <= '1' When BusCycle_Sig <"01110" Else '0';
    -- On AppleII_Reset, switch clock to free running 1MHz clock so the cpu has a chance to reset itself, otherwise use clock derived from buscycle
    cpuclk_6502 <= AuxClk When AppleII_Reset='1'      Else
                   '1'    When BusCycle_Sig < "01110" Else
                   '0';
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If ClockCounter = "11011" Then
                AuxClk <= Not AuxClk;
                ClockCounter <= (Others=>'0');
            Else
                ClockCounter <= ClockCounter + 1;
            End If;
        End If;
    End Process;

    memory_access_time <= BusCycleIs3 Or BusCycleIs2 Or BusCycleIs17 Or BusCycleIs16 Or BusCycleIs7 Or BusCycleIs8;
--    do_store_m <= do_store And TextIn;
    cpu_ram_oe     <= cpu_access_time And rw_6502;
    ram_card_read  <= address_is_rom  And card_ram_rd_sig;
    ram_card_write <= address_is_rom  And card_ram_we_sig;

    low200       <= '1' When cpu_addr(15 Downto  9) = "0000000" Else '0';
    c000_up      <= '1' When cpu_addr(15 Downto 14) = "11"      Else '0';
    text_page0   <= '1' When cpu_addr(15 Downto 10) = "000001"  Else '0';
    hgr_page0    <= '1' When cpu_addr(15 Downto 13) = "001"     Else '0';
    video_memory <= mode_80STORE_Sig And (text_page0 Or (HiRes And hgr_page0));
    auxilary_mem <= ALTZP     When ((ram_card_read='1' And rw_6502='1') Or (ram_card_write='1' And rw_6502='0') Or low200='1') Else
                    PAGE2_Sig When video_memory='1'                                                                            Else
                    RAMRD_Sig When mem_rw_6502='1'                                                                             Else
                    RAMWRT_Sig;

    ram_ub     <= Not (auxilary_mem Or (LDPS And mode_80COL));
    ram_lb_Sig <= Not (Not auxilary_mem Or LDPS);

    RAM_address <= "00" & apple_video_addr When LDPS='1' Else
                   "00" & final_ram_addr;

    MMU_bit     <= readback_mmu;
--    p <= pixelout;

    n_OE          <= Not (LDPS Or cpu_ram_oe);
    disk_cycle    <= BusCycleIs5 Or BusCycleIs6;
    RAM_access    <= address_is_ram Or ram_card_read Or ram_card_write;
    n_WE          <= Not (Not mem_rw_6502 And (address_is_ram Or ram_card_write));
    n_CS          <= Not (RAM_access Or LDPS);
    io_access_sig <= '1' When cpu_addr(15 Downto 8)="11000000" Else '0';
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(cpu_addr, bank1_sig, card_ram_rd_sig, RAMRD_Sig, RAMWRT_Sig, INTCXROM_Sig, ALTZP, SLOTC3ROM_Sig, mode_80STORE_Sig, VBL, TextIn, Mixed, PAGE2_Sig, HiRes, AltCharSet, mode_80COL)
    Begin
        Case cpu_addr(3 Downto 0) Is
            When   x"0" => readback_mmu <= '0';
            When   x"1" => readback_mmu <= Not bank1_sig;
            When   x"2" => readback_mmu <= card_ram_rd_sig;
            When   x"3" => readback_mmu <= RAMRD_Sig;
            When   x"4" => readback_mmu <= RAMWRT_Sig;
            When   x"5" => readback_mmu <= INTCXROM_Sig;
            When   x"6" => readback_mmu <= ALTZP;
            When   x"7" => readback_mmu <= SLOTC3ROM_Sig;
            When   x"8" => readback_mmu <= mode_80STORE_Sig;
            When   x"9" => readback_mmu <= Not VBL;
            When   x"A" => readback_mmu <= TextIn;
            When   x"B" => readback_mmu <= Mixed;
            When   x"C" => readback_mmu <= PAGE2_Sig;
            When   x"D" => readback_mmu <= HiRes;
            When   x"E" => readback_mmu <= AltCharSet;
            When Others => readback_mmu <= mode_80COL;
        End Case;
    End Process;
    -- ------------------------------------------------------------------------


    -- ------------------------------------------------------------------------
    Process(clk28)
    Begin
        If Rising_Edge(clk28) Then
            If AppleII_Reset='1' Then
                PAGE2_Sig        <= '0';
                HiRes            <= '0';
                AltCharSet       <= '0';
                mode_80COL       <= '0';
                TextIn           <= '1';
                INTCXROM_Sig     <= '0';
                INTC8ROM         <= '0';
                SLOTC3ROM_Sig    <= '0';
                RAMRD_Sig        <= '0';
                RAMWRT_Sig       <= '0';
                mode_80STORE_Sig <= '0';
                ALTZP            <= '0';
            Else
                If (BusCycleIs5='1' And io_access_sig='1') Then
                    Case (cpu_addr(7 Downto 0)) Is
                        When x"50" => TextIn    <= '0';
                        When x"51" => TextIn    <= '1';

                        When x"52" => Mixed     <= '0';
                        When x"53" => Mixed     <= '1';

                        When x"54" => PAGE2_Sig <= '0';
                        When x"55" => PAGE2_Sig <= '1';

                        When x"56" => HiRes     <= '0';
                        When x"57" => HiRes     <= '1';
                        When Others=> Null;
                    End Case;

                    If (cpu_addr(7 Downto 0) = x"30") Then
                        speaker <= Not speaker;
                    End If;

                    If (rw_6502='0') Then
                        Case cpu_addr(7 Downto 0) Is
                            When x"00" => mode_80STORE_Sig <= '0'; -- $C000
                            When x"01" => mode_80STORE_Sig <= '1';
                            When x"02" => RAMRD_Sig <= '0';
                            When x"03" => RAMRD_Sig <= '1';
                            When x"04" => RAMWRT_Sig <= '0';
                            When x"05" => RAMWRT_Sig <= '1';
                            When x"06" => INTCXROM_Sig <= '0';
                            When x"07" => INTCXROM_Sig <= '1';
                            When x"08" => ALTZP <= '0';
                            When x"09" => ALTZP <= '1';
                            When x"0A" => SLOTC3ROM_Sig <= '0';
                            When x"0B" => SLOTC3ROM_Sig <= '1';
                            When x"0C" => mode_80COL <= '0';
                            When x"0D" => mode_80COL <= '1';
                            When x"0E" => AltCharSet <= '0';
                            When x"0F" => AltCharSet <= '1';
                            When Others => Null;
                        End Case;
                    End If;
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    Notrw_6502 <= Not rw_6502;
    -- ------------------------------------------------------------------------
    U_RamCard : ramcard Port Map (
        mclk28      => clk28,
        reset_in    => AppleII_Reset,
        strobe      => BusCycleIs7,
        addr        => cpu_addr,
        ram_addr    => final_ram_addr,
        we          => Notrw_6502,
        card_ram_we => card_ram_we_Sig,
        card_ram_rd => card_ram_rd_sig,
        bank1       => bank1_sig
    );
    -- ------------------------------------------------------------------------

    doshift <= Not BusCycle_sig(0) Or mode_80COL;
    odd_dot <= BusCycle_Sig(1);
    -- ------------------------------------------------------------------------
    U_AppleShifter : AppleShifter Port Map (
        pixelout      => pixelout,
        AppleII_Reset => AppleII_Reset,
        GR            => GR,
        char_ROM_addr => char_ROM_addr,
        vga_R_r       => vga_R,
        vga_G_r       => vga_G,
        vga_B_r       => vga_B,
        clk28         => clk28,
        do_shift      => doshift,
        mode_80COL    => mode_80COL,
        odd_dot       => odd_dot,
        LDPS          => LDPS,
        aux_LDPS      => aux_LDPS,
        TextIn        => TextIn,
        Mixed         => Mixed,
        AltCharSet    => AltCharSet,
        HiRes         => HiRes,
        flash_count4  => flash_count4,
        HBL           => HBL,
        VBL           => VBL,
        ram_data      => ram_data,
        CounterY      => CounterY,
        char_ROM_data => char_ROM_data
    );
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    U_AppleVideoTiming : AppleVideoTiming Port Map (
        -- Outputs
        vga_HS        => vga_HS,
        vga_VS        => vga_VS,
        HBL           => HBL,
        VBL           => VBL,
        LDPS          => LDPS,
        BusCycle      => BusCycle_Sig,
        flash_count4  => flash_count4,
        CounterX      => CounterX,
        CounterY      => CounterY,
        -- Inputs
        AppleII_Reset => AppleII_Reset,
--        reset_in => reset_in
        clk28         => clk28
      );

    -- ------------------------------------------------------------------------
    -- ------------------------------------------------------------------------
    -- ------------------------------------------------------------------------
    -- ------------------------------------------------------------------------
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
