-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity CpuInSelect Is
    Port (
        CPU_DataOut         : Out Std_Logic_Vector( 7 Downto 0);
        CPU_Address         : In  Std_Logic_Vector(15 Downto 0);
        KBD_DataIn          : In  Std_Logic_Vector( 7 Downto 0);
        RAM_DataIn          : In  Std_Logic_Vector( 7 Downto 0);
        IDE_DataIn          : In  Std_Logic_Vector( 7 Downto 0);
--        AppleII_Buttons     : In  Std_Logic_Vector( 1 Downto 0);
        AppleII_ButtonLeft  : In  Std_Logic;
        AppleII_ButtonRight : In  Std_Logic;
        AppleII_SelectC7Io  : Out Std_Logic;
        Card_Ram_Read       : In  Std_Logic;
        ROM_DataIn          : In  Std_Logic_Vector( 7 Downto 0);
        ROM_Address         : Out Std_Logic_Vector(13 Downto 0);
        MMU_Bit             : In  Std_Logic;
        KBD_Read            : Out Std_Logic;

        INTC8ROM            : In Std_Logic;
        SLOTC3ROM           : In Std_Logic;
        INTCXROM            : In Std_Logic
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of CpuInSelect Is
-- ----------------------------------------------------------------------------
    Signal SelectRomSpace           : Std_Logic;
    Signal SelectSlotsRomSpace      : Std_Logic;
    Signal SelectInternalC8Rom      : Std_Logic;
    Signal SelectRamSpace           : Std_Logic;
    Signal SelectC7Io               : Std_Logic;
    Signal SelectC7Rom              : Std_Logic;
    Signal SelectRomDataIn          : Std_Logic;
    Signal SelectRamDataIn          : Std_Logic;
    Signal SelectIdeDataIn          : Std_Logic;
    Signal SelectMMUBit             : Std_Logic;
    Signal SelectAppleIIButtonLeft  : Std_Logic;
    Signal SelectAppleIIButtonRight : Std_Logic;
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    SelectRomSpace      <= CPU_Address(15) And CPU_Address(14) And (CPU_Address(13) Or CPU_Address(12));
    SelectSlotsRomSpace <= '1' When (CPU_Address(15 Downto 8) > x"C0") And  (CPU_Address(15 Downto 8) < x"C8") Else '0';
    SelectRamSpace      <= Not CPU_Address(15) Or Not CPU_Address(14);
    ROM_Address         <= "000000" & CPU_Address(7 Downto 0) When (SelectC7Rom='1' And INTCXROM='0') Else CPU_Address(13 Downto 0);

    SelectRomDataIn          <= (SelectRomSpace And Not Card_Ram_Read) Or SelectInternalC8Rom Or SelectSlotsRomSpace;
    SelectRamDataIn          <= SelectRamSpace Or (Card_Ram_Read And SelectRomSpace);
    SelectIdeDataIn          <= SelectC7Io;
    KBD_Read                 <= '1' When CPU_Address = x"C010" Else '0';
    SelectInternalC8Rom      <= '1' When CPU_Address(15 Downto 11) = "11001" Else '0'; -- 0xC888-0xCFFF
    SelectC7Io               <= '1' When CPU_Address(15 Downto  4) = x"C0F"  Else '0'; -- c0fxx
    SelectC7Rom              <= '1' When CPU_Address(15 Downto  8) = x"C7"   Else '0'; -- c7xx
    SelectMMUBit             <= '1' When CPU_Address(15 Downto  4) = x"C01"  Else '0';
    SelectAppleIIButtonLeft  <= '1' When CPU_Address               = x"C061" Else '0';
    SelectAppleIIButtonRight <= '1' When CPU_Address               = x"C062" Else '0';


    CPU_DataOut <= ROM_DataIn                      When SelectRomDataIn         ='1' Else
                   RAM_DataIn                      When SelectRamDataIn         ='1' Else
                   IDE_DataIn                      When SelectIdeDataIn         ='1' Else    -- $COF
                   MMU_Bit             & "0000001" When SelectMMUBit            ='1' Else
                   AppleII_ButtonLeft  & "0000000" When SelectAppleIIButtonLeft ='1' Else
                   AppleII_ButtonRight & "0000000" When SelectAppleIIButtonRight='1' Else
                   KBD_DataIn;

    AppleII_SelectC7Io <= SelectC7Io;

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------

