-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
USe     Work.AsciiTranslation.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity KBD_Translate Is
    Port (
        Shift   : In  Std_Logic;
        InCode  : In  Std_Logic_Vector(7 Downto 0);
        OutCode : Out Std_Logic_Vector(6 Downto 0)
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of KBD_Translate Is
-- ----------------------------------------------------------------------------

    Signal Ascii            : Std_Logic_Vector(7 Downto 0);
    Signal ShiftKeyPlusCode : Std_Logic_Vector(11 Downto 0);
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    ShiftKeyPlusCode <= "000" & Shift & InCode;
    OutCode          <= Ascii(6 Downto 0);
    -- ------------------------------------------------------------------------
    Process(ShiftKeyPlusCode)
    Begin
        Case ShiftKeyPlusCode Is
            When cKey_066 | cKey_166 => Ascii <= cAscii_08;    -- backspace key
            When cKey_00D | cKey_10D => Ascii <= cAscii_09;    -- horizontal Tab
            When cKey_05A | cKey_15A => Ascii <= cAscii_0D;    -- Carriage return ('enter' key)
            When cKey_076 | cKey_176 => Ascii <= cAscii_1B;    -- Escape ('esc' key)
            When cKey_029 | cKey_129 => Ascii <= cAscii_20;    -- Space

            When cKey_116 => Ascii <= cAscii_21; -- !
            When cKey_152 => Ascii <= cAscii_22; -- "
            When cKey_126 => Ascii <= cAscii_23; -- #
            When cKey_125 => Ascii <= cAscii_24; -- $
            When cKey_12E => Ascii <= cAscii_25; --
            When cKey_13D => Ascii <= cAscii_26; --
            When cKey_052 => Ascii <= cAscii_27; --
            When cKey_146 => Ascii <= cAscii_28; --
            When cKey_145 => Ascii <= cAscii_29; --
            When cKey_13E => Ascii <= cAscii_2A; -- *
            When cKey_155 => Ascii <= cAscii_2B; -- +
            When cKey_041 => Ascii <= cAscii_2C; -- '
            When cKey_04E => Ascii <= cAscii_2D; -- -
            When cKey_049 => Ascii <= cAscii_2E; -- .
            When cKey_04A => Ascii <= cAscii_2F; -- /
            When cKey_045 => Ascii <= cAscii_30; -- 0
            When cKey_016 => Ascii <= cAscii_31; -- 1
            When cKey_01E => Ascii <= cAscii_32; -- 2
            When cKey_026 => Ascii <= cAscii_33; -- 3
            When cKey_025 => Ascii <= cAscii_34; -- 4
            When cKey_02E => Ascii <= cAscii_35; -- 5
            When cKey_036 => Ascii <= cAscii_36; -- 6
            When cKey_03D => Ascii <= cAscii_37; -- 7
            When cKey_03E => Ascii <= cAscii_38; -- 8
            When cKey_046 => Ascii <= cAscii_39; -- 9
            When cKey_14C => Ascii <= cAscii_3A; -- :
            When cKey_04C => Ascii <= cAscii_3B; -- ;
            When cKey_141 => Ascii <= cAscii_3C; -- <
            When cKey_055 => Ascii <= cAscii_3D; -- =
            When cKey_149 => Ascii <= cAscii_3E; -- >
            When cKey_14A => Ascii <= cAscii_3F; -- ?
            When cKey_11E => Ascii <= cAscii_40; -- @
            When cKey_11C => Ascii <= cAscii_41; -- A
            When cKey_132 => Ascii <= cAscii_42; -- B
            When cKey_121 => Ascii <= cAscii_43; -- C
            When cKey_123 => Ascii <= cAscii_44; -- D
            When cKey_124 => Ascii <= cAscii_45; -- E
            When cKey_12B => Ascii <= cAscii_46; -- F
            When cKey_134 => Ascii <= cAscii_47; -- G
            When cKey_133 => Ascii <= cAscii_48; -- H
            When cKey_143 => Ascii <= cAscii_49; -- I
            When cKey_13B => Ascii <= cAscii_4A; -- J
            When cKey_142 => Ascii <= cAscii_4B; -- K
            When cKey_14B => Ascii <= cAscii_4C; -- L
            When cKey_13A => Ascii <= cAscii_4D; -- M
            When cKey_131 => Ascii <= cAscii_4E; -- N
            When cKey_144 => Ascii <= cAscii_4F; -- O
            When cKey_14D => Ascii <= cAscii_50; -- P
            When cKey_115 => Ascii <= cAscii_51; -- Q
            When cKey_12D => Ascii <= cAscii_52; -- R
            When cKey_11B => Ascii <= cAscii_53; -- S
            When cKey_12C => Ascii <= cAscii_54; -- T
            When cKey_13C => Ascii <= cAscii_55; -- U
            When cKey_12A => Ascii <= cAscii_56; -- V
            When cKey_11D => Ascii <= cAscii_57; -- W
            When cKey_122 => Ascii <= cAscii_58; -- Y
            When cKey_135 => Ascii <= cAscii_59; -- X
            When cKey_11A => Ascii <= cAscii_5A; -- Z
            When cKey_054 => Ascii <= cAscii_5B; -- [
            When cKey_05D => Ascii <= cAscii_5C; -- \
            When cKey_05B => Ascii <= cAscii_5D; -- ]
            When cKey_136 => Ascii <= cAscii_5E; -- ^
            When cKey_14E => Ascii <= cAscii_5F; -- _
            When cKey_00E => Ascii <= cAscii_60; -- `
            When cKey_01C => Ascii <= cAscii_61; -- a
            When cKey_032 => Ascii <= cAscii_62; -- b
            When cKey_021 => Ascii <= cAscii_63; -- c
            When cKey_023 => Ascii <= cAscii_64; -- d
            When cKey_024 => Ascii <= cAscii_65; -- e
            When cKey_02B => Ascii <= cAscii_66; -- f
            When cKey_034 => Ascii <= cAscii_67; -- g
            When cKey_033 => Ascii <= cAscii_68; -- h
            When cKey_043 => Ascii <= cAscii_69; -- i
            When cKey_03B => Ascii <= cAscii_6A; -- j
            When cKey_042 => Ascii <= cAscii_6B; -- k
            When cKey_04B => Ascii <= cAscii_6C; -- l
            When cKey_03A => Ascii <= cAscii_6D; -- m
            When cKey_031 => Ascii <= cAscii_6E; -- n
            When cKey_044 => Ascii <= cAscii_6F; -- o
            When cKey_04D => Ascii <= cAscii_70; -- p
            When cKey_015 => Ascii <= cAscii_71; -- q
            When cKey_02D => Ascii <= cAscii_72; -- r
            When cKey_01B => Ascii <= cAscii_73; -- s
            When cKey_02C => Ascii <= cAscii_74; -- t
            When cKey_03C => Ascii <= cAscii_75; -- u
            When cKey_02A => Ascii <= cAscii_76; -- v
            When cKey_01D => Ascii <= cAscii_77; -- w
            When cKey_022 => Ascii <= cAscii_78; -- x
            When cKey_035 => Ascii <= cAscii_79; -- y
            When cKey_01A => Ascii <= cAscii_7A; -- z
            When cKey_154 => Ascii <= cAscii_7B; -- {
            When cKey_15D => Ascii <= cAscii_7C; -- |
            When cKey_15B => Ascii <= cAscii_7D; -- }
            When cKey_10E => Ascii <= cAscii_7E; -- ~
            When cKey_071 => Ascii <= cAscii_7F; -- Delete or DEL on numeric keypad
            When cKey_074 => Ascii <= cAscii_15; -- right arrow (cntrl U)
            When cKey_06B => Ascii <= cAscii_08; -- left arrow (BS)
            When cKey_075 => Ascii <= cAscii_0B; -- up arrow
            When cKey_072 => Ascii <= cAscii_0A; -- down arrow, ^J, LF
            When cKey_171 => Ascii <= cAscii_7F; -- Delete or DEL on numeric keypad
            When Others   => AScii <= cAscii_00; -- 0x00 for unlisted characters
        End Case;
    End Process;
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
