-- RAF: Original file header
-- File translated with vhd2vl v 1.0
-- VHDL to Verilog RTL translator
-- Copyright (C) 2001 Vincenzo Liguori - Ocean Logic Pty Ltd - http://www.ocean-logic.com
-- vhd2vl comes with ABSOLUTELY NO WARRANTY
-- This is free software, and you are welcome to redistribute it
-- under certain conditions.
-- See the license file license.txt included with the source for details.
--
-- PS2_Ctrl.vhd
-- ------------------------------------------------
--   Simplified PS/2 Controller  (kbd, mouse...)
-- ------------------------------------------------
-- Only the Receive function is implemented !
-- (c) ALSE. http://www.alse-fr.com
--
--  RAF: VHDL translation for 65C02 simulation in VHDL
--
-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
Use     IEEE.Std_Logic_Unsigned.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity PS2_Controller Is
    Port (
        Clk       : In  Std_Logic;
        Reset     : In  Std_Logic;
        PS2_CLK   : In  Std_Logic;
        PS2_Data  : In  Std_Logic;
        DoRead    : In  Std_Logic;
        Scan_Err  : Out Std_Logic;
        Scan_DAV  : Out Std_Logic;
        Scan_Code : Out Std_Logic_Vector(7 Downto 0)
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of PS2_Controller Is
-- ----------------------------------------------------------------------------
    Constant sIdle     : Std_Logic_Vector(1 Downto 0) := "00";
    Constant sShifting : Std_Logic_Vector(1 Downto 0) := "01";

    Signal PS2_Datr  : Std_Logic;
    Signal PS2_Clk_f : Std_Logic;
    Signal Filter    : Std_Logic_Vector(7 Downto 0);
    Signal Fall_Clk  : Std_Logic;
    Signal State     : Std_Logic_Vector(1 Downto 0);
    Signal Bit_Cnt   : Std_Logic_Vector(3 Downto 0);
    Signal S_Reg     : Std_Logic_Vector(8 Downto 0);
    Signal Parity    : Std_Logic;
    Signal Scan_DAVi : Std_Logic;
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    Scan_DAV <= Scan_DAVi;
    -- ------------------------------------------------------------------------
    Process(Clk, Reset)
    Begin
        If Reset='1' Then
            PS2_Datr  <= '0';
            PS2_Clk_f <= '0';
            Filter    <= (Others=>'0');
            Fall_Clk  <= '0';
        Else
            If Rising_Edge(Clk) Then
                PS2_Datr <= PS2_Data;
                Fall_Clk <= '0';
                Filter  <= PS2_CLK & Filter(7 Downto 1);

                If (Filter(0)='1') Then
--                If (Filter=x"FF") Then   --RAF: analyze this circuit again.
                    PS2_Clk_f <= '1';
                ElsIf (Filter(0) = '0') Then
--                ElsIf (Filter = x"00") Then
                    PS2_Clk_f <= '0';
                    If PS2_Clk_f = '1' Then
                        Fall_Clk <= '1';
                    End If;
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(Clk, Reset)
    Begin
        If Reset='1' Then
            State     <= sIdle;
            Bit_Cnt   <= (Others=>'0');
            S_Reg     <= (Others=>'0');
            Scan_Code <= (Others=>'0');
            Parity    <= '0';
            Scan_DAVi <= '0';
            Scan_Err  <= '0';
        Else
            If Rising_Edge(Clk) Then
                If DoRead='1' Then
                    Scan_DAVi <= '0';
                End If;

                Case State Is
                    When sIdle =>
                         Parity  <= '0';
                         Bit_Cnt <= (Others=>'0');
                         If (Fall_Clk='1' And PS2_Datr='0') Then
                            Scan_Err <= '0';
                            State    <= sShifting;
                         End If;
                    When sShifting =>
                         If (Bit_Cnt >= "1001") Then
                            If (Fall_Clk = '1') Then
                                Scan_Err  <= Not Parity Or Not PS2_Datr Or Scan_DAVi;
                                Scan_DAVi <= '1';
                                Scan_Code <= S_Reg(7 Downto 0);
                                State     <= sIdle;
                            End If;
                         Else
                            If (Fall_Clk = '1') Then
                                Bit_Cnt <= Bit_Cnt + 1;
                                S_Reg   <= PS2_Datr & S_Reg(8 Downto 1);
                                Parity  <= Parity Xor PS2_Datr;
                            End If;
                         End If;
                    When Others =>
                        State <= sIdle;
                End Case;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
