-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity lores_color_map Is
    Port (
        nibble  : In  Std_Logic_Vector(3 Downto 0);
        RGB_val : Out Std_Logic_Vector(11 Downto 0)
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of lores_color_map Is
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

--/*
--Color    Color Register    Master Color
--Name         Value            Value
-----------------------------------------
--Black         $0              $0000
--Deep Red      $1              $0D03
--Dark Blue     $2              $0009
--Purple        $3              $0D2D
--Dark Green    $4              $0072
--Dark Gray     $5              $0555
--Medium Blue   $6              $022F
--Light Blue    $7              $06AF
--Brown         $8              $0850
--Orange        $9              $0F60
--Light Gray    $A              $0AAA
--Pink          $B              $0F98
--Light Green   $C              $01D0
--Yellow        $D              $0FF0
--Aquamarine    $E              $04F9
--White         $F              $0FFF
-----------------------------------------
--*/

    -- ------------------------------------------------------------------------
    Process(nibble)
    Begin
        Case nibble Is
            When   x"0" => RGB_val <= x"000";
            When   x"1" => RGB_val <= x"D03";
            When   x"2" => RGB_val <= x"009";
            When   x"3" => RGB_val <= x"D2D";
            When   x"4" => RGB_val <= x"072";
            When   x"5" => RGB_val <= x"555";
            When   x"6" => RGB_val <= x"22f";
            When   x"7" => RGB_val <= x"6af";
            When   x"8" => RGB_val <= x"850";
            When   x"9" => RGB_val <= x"f60";
            When   x"A" => RGB_val <= x"aaa";
            When   x"B" => RGB_val <= x"f98";
            When   x"C" => RGB_val <= x"1d0";
            When   x"D" => RGB_val <= x"ff0";
            When   x"E" => RGB_val <= x"4f9";
            When Others => RGB_val <= x"fff";
        End Case;
    End Process;
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
