-- ----------------------------------------------------------------------------
Library IEEE;
Use     IEEE.Std_Logic_1164.All;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Entity RamCard Is
    Port (
        mclk28      : In  Std_Logic;
        reset_in    : In  Std_Logic;
        strobe      : In  Std_Logic;
        addr        : In  Std_Logic_Vector(15 Downto 0);
        ram_addr    : Out Std_Logic_Vector(15 Downto 0);
        we          : In  Std_Logic;
        card_ram_we : Out Std_Logic;
        card_ram_rd : Out Std_Logic;
        bank1       : Out Std_Logic
    );
End;
-- ----------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
Architecture RTL Of RamCard Is
-- ----------------------------------------------------------------------------
    Signal bank1_sig : Std_Logic;
    Signal read_en   : Std_logic;
    Signal write_en  : Std_Logic;
    Signal pre_wr_en : Std_Logic;
    Signal Dxxx      : Std_Logic;
    Signal addr12    : Std_Logic;
-- ----------------------------------------------------------------------------
Begin
-- ----------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    Process(mclk28)
    Begin
        If Rising_Edge(mclk28) Then
            If reset_in='1' Then
                bank1_sig <= '0';
                read_en   <= '0';
                write_en  <= '1';
--                write_en  <= '0';   -- RAF:
                pre_wr_en <= '0';
            Else
                If ((addr(15 Downto 4) = x"C08") And strobe='1') Then
                    bank1_sig <= addr(3);
                    If (we='0') Then
                        pre_wr_en <= addr(0);
                    Else
                        pre_wr_en <= '0';
                    End If;
                    If (addr(0) = '0') Then
                        write_en <= '0';
                    Else
                        If (pre_wr_en='1' And we='0') Then
                            write_en <= '1';
                        End If;
                    End If;
                    read_en <= Not (addr(0) Xor Addr(1));
                End If;
            End If;
        End If;
    End Process;
    -- ------------------------------------------------------------------------

    -- ------------------------------------------------------------------------
    bank1       <= bank1_sig;
    Dxxx        <= '1' When (addr(15 Downto 12) = "1101") Else '0';
    addr12      <= addr(12) And Not (bank1_sig And Dxxx);
    ram_addr    <= addr(15 Downto 13) & addr12 & addr(11 Downto 0);
    card_ram_we <= write_en;
    card_ram_rd <= read_en;
    -- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------
End RTL;
-- ----------------------------------------------------------------------------
